package cachedata

import (
	"context"
	lru "github.com/hashicorp/golang-lru"
	"github.com/odysseus/payment/model"
	goredislib "github.com/redis/go-redis/v9"
)

type CacheData struct {
	rdb     *goredislib.Client
	pathLru *lru.Cache
	ctx     context.Context
	cancel  context.CancelFunc

	taskType        model.TaskTypeRepository
	userRepo        model.UserRepository
	userLevel       model.UserLevelRepository
	userAndTaskRepo model.UserLevelTaskTypeRepository
}

func NewCacheData(ctx context.Context, options goredislib.Options) *CacheData {
	rdb := goredislib.NewClient(&options)
	pathLru, _ := lru.New(10000)
	cache := &CacheData{
		rdb:             rdb,
		pathLru:         pathLru,
		taskType:        model.NewTaskTypeRepository(),
		userRepo:        model.NewUserRepository(),
		userLevel:       model.NewUserLevelRepository(),
		userAndTaskRepo: model.NewUserLevelTaskTypeRepository(),
	}
	cache.ctx, cache.cancel = context.WithCancel(ctx)
	return cache
}
