package payment

import (
	"context"
	"fmt"
	"github.com/stretchr/testify/assert"
	"testing"
)

func TestPayment(t *testing.T) {
	// Initialize a new Payment instance
	p := NewPayment(RedisConnParam{
		Addr:     "127.0.0.1:6379",
		Password: "123456",
		DbIndex:  0,
	})

	t.Run("Test Balance", func(t *testing.T) {
		// Test IncrBalance
		_, err := p.IncrBalance(context.Background(), "testuser", 100)
		assert.NoError(t, err)

		// Test GetBalance
		balance, err := p.GetBalance(context.Background(), "testuser")
		assert.NoError(t, err)
		assert.Equal(t, int64(100), balance)

		// Test DecrBalance
		_, err = p.DecrBalance(context.Background(), "testuser", 50)
		assert.NoError(t, err)
	})

	t.Run("Test Charge", func(t *testing.T) {
		// Test IncrCharge
		_, err := p.IncrCharge(context.Background(), "testuser", 200)
		assert.NoError(t, err)

		// Test GetCharge
		charge, err := p.GetCharge(context.Background(), "testuser")
		assert.NoError(t, err)
		assert.Equal(t, int64(200), charge)

		// Test DecrCharge
		_, err = p.DecrCharge(context.Background(), "testuser", 100)
		assert.NoError(t, err)
	})

	t.Run("Test Credits", func(t *testing.T) {
		// Test IncrCredits
		_, err := p.IncrCredits(context.Background(), "testuser", 300)
		assert.NoError(t, err)

		// Test GetCredits
		credits, err := p.GetCredits(context.Background(), "testuser")
		assert.NoError(t, err)
		assert.Equal(t, int64(300), credits)

		// Test DecrCredits
		_, err = p.DecrCredits(context.Background(), "testuser", 150)
		assert.NoError(t, err)
	})

	// Close the Payment instance
	err := p.Close()
	assert.NoError(t, err)
}

// add benchmark test for IncrBalance
func BenchmarkPaymentIncrBalance(b *testing.B) {
	p := NewPayment(RedisConnParam{
		Addr:     "127.0.0.1:6379",
		Password: "123456",
		DbIndex:  0,
	})
	defer p.Close()

	// run the IncrBalance function b.N times with different uid

	for n := 0; n < b.N; n++ {
		uid := fmt.Sprintf("11%d", n)
		_, err := p.IncrBalance(context.Background(), uid, 100)
		if err != nil {
			b.Error(err)
		}
	}

}
